"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var request = require('request');
var inversify_1 = require('inversify');
var path = require('path');
var fs = require('fs');
var nodeUrl = require('url');
var _ = require('lodash');
var force_error_impl_1 = require("./force-error-impl");
var fileExists = require('file-exists');
var safeJsonParse = require('safe-json-parse/callback');
var async = require('async');
var RemoteCatalogGetterImpl = (function (_super) {
    __extends(RemoteCatalogGetterImpl, _super);
    function RemoteCatalogGetterImpl(commandUtil) {
        _super.call(this);
        this.commandUtil = commandUtil;
    }
    RemoteCatalogGetterImpl.prototype.getCatalogFromUrl = function (url, cb) {
        var me = this;
        var parsedUrl = RemoteCatalogGetterImpl.getParsedUrl(url);
        if (parsedUrl.protocol) {
            me.baseUrl = parsedUrl.protocol + "//" + parsedUrl.hostname + path.dirname(parsedUrl.path);
        }
        else {
            me.baseUrl = path.dirname(parsedUrl.path);
        }
        me.getRemoteResource(parsedUrl, 'root', function (err, remoteCatalogResource) {
            if (me.commandUtil.callbackIfError(cb, err)) {
                return;
            }
            try {
                var remoteCatalog_1 = remoteCatalogResource.parsedObject;
                var fnArray_1 = [];
                remoteCatalog_1.entries.forEach(function (entry) {
                    entry.urls.forEach(function (url) {
                        var parsedUrl = RemoteCatalogGetterImpl.getParsedUrl(url);
                        if (!parsedUrl.protocol) {
                            url = path.isAbsolute(parsedUrl.path) ? parsedUrl.path : me.baseUrl + "/" + parsedUrl.path;
                        }
                        fnArray_1.push(async.apply(me.getRemoteResource.bind(me), url, entry.name));
                    });
                });
                async.parallel(fnArray_1, function (err, results) {
                    remoteCatalog_1.entries.forEach(function (entry) {
                        entry.resources =
                            _.filter(results, ['parentCatalogEntryName', entry.name]);
                    });
                    cb(null, remoteCatalog_1);
                });
            }
            catch (err) {
                me.commandUtil.callbackIfError(cb, err);
            }
        });
    };
    RemoteCatalogGetterImpl.prototype.getRemoteResource = function (url, parentCatalogEntryName, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        me.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            if (me.commandUtil.callbackIfError(cb, err)) {
                return;
            }
            safeJsonParse(text, function (err, parsedObject) {
                var name = path.basename(absoluteUrl);
                cb(null, {
                    absoluteUrl: absoluteUrl, name: name, text: text, parsedObject: parsedObject, parentCatalogEntryName: parentCatalogEntryName
                });
            });
        });
    };
    RemoteCatalogGetterImpl.prototype.resolveJsonObjectFromUrl = function (url, cb) {
        var me = this;
        cb = cb || (function () {
        });
        me.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            safeJsonParse(text, function (err, jsonObject) {
                cb(err, jsonObject, absoluteUrl);
            });
        });
    };
    RemoteCatalogGetterImpl.prototype.resolveTextResourceFromUrl = function (url, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        try {
            var parsedUrl_1 = RemoteCatalogGetterImpl.getParsedUrl(url);
            if (!parsedUrl_1) {
                cb(new Error("Url: '" + url + "' is unusable"));
                return;
            }
            if (!parsedUrl_1.protocol) {
                var urlString = parsedUrl_1.path;
                if (!fileExists(urlString)) {
                    cb(new Error(urlString + " doesn't exist"), null);
                    return;
                }
                cb(null, fs.readFileSync(urlString, 'utf8'), urlString);
                return;
            }
            request(parsedUrl_1.href, function (err, res, text) {
                if (res.statusCode !== 200) {
                    cb(new Error("Error retrieving '" + parsedUrl_1.href + "'"), null);
                    return;
                }
                cb(null, text, parsedUrl_1.href);
            });
        }
        catch (err) {
            cb(err, null);
        }
    };
    RemoteCatalogGetterImpl.getParsedUrl = function (url) {
        try {
            if (url.constructor.name === 'Url') {
                return url;
            }
            else if (url.constructor.name === 'String') {
                return nodeUrl.parse(url);
            }
        }
        catch (err) {
            return null;
        }
    };
    RemoteCatalogGetterImpl = __decorate([
        __param(0, inversify_1.inject('CommandUtil')), 
        __metadata('design:paramtypes', [Object])
    ], RemoteCatalogGetterImpl);
    return RemoteCatalogGetterImpl;
}(force_error_impl_1.ForceErrorImpl));
exports.RemoteCatalogGetterImpl = RemoteCatalogGetterImpl;
//# sourceMappingURL=remote-catalog-impl.js.map